#include "colorgradient.h"
#include "base/math/interpolation.h"
#include "base/math/range.h"
#include "base/text/smartstring.h"

#include "base/math/segmentedfunc.tpp"

template struct Math::SegmentedFunction<Gfx::Color>;

using namespace Gfx;

ColorGradient::ColorGradient(const std::string &stoplist)
{
	auto stopStrings = Text::SmartString::split(stoplist, ',', true);
	auto pos = 0.0;
	for (auto &stopString : stopStrings)
	{
		auto parts = Text::SmartString::split(stopString, ' ', true);
		if (parts.size() == 2) {
			pos = std::stod(parts[1]);
			stops.push_back({ pos, Color(parts[0]) });
		}
		else throw std::logic_error("invalid gradient stop: " + stopString);
	}
}

ColorGradient::operator std::string() const
{
	std::string res;
	for (auto &stop : stops)
	{
		if (!res.empty()) res += ", ";
		res += (std::string)stop.value + " " + std::to_string(stop.pos);
	}
	return res;
}

ColorGradient ColorGradient::HeatMap5Color()
{
	ColorGradient res;
	res.stops.push_back({ 0.0 / 4.0, Gfx::Color(0.0, 0.0, 1.0) });
	res.stops.push_back({ 1.0 / 4.0, Gfx::Color(0.0, 1.0, 1.0) });
	res.stops.push_back({ 2.0 / 4.0, Gfx::Color(0.0, 1.0, 0.0) });
	res.stops.push_back({ 3.0 / 4.0, Gfx::Color(1.0, 1.0, 0.0) });
	res.stops.push_back({ 4.0 / 4.0, Gfx::Color(1.0, 0.0, 0.0) });
	return res;
}

ColorGradient ColorGradient::HeatMap7Color()
{
	ColorGradient res;
	res.stops.push_back({ 0.0 / 6.0, Gfx::Color(0.0, 0.0, 0.0) });
	res.stops.push_back({ 1.0 / 6.0, Gfx::Color(0.0, 0.0, 1.0) });
	res.stops.push_back({ 2.0 / 6.0, Gfx::Color(0.0, 1.0, 1.0) });
	res.stops.push_back({ 3.0 / 6.0, Gfx::Color(0.0, 1.0, 0.0) });
	res.stops.push_back({ 4.0 / 6.0, Gfx::Color(1.0, 1.0, 0.0) });
	res.stops.push_back({ 5.0 / 6.0, Gfx::Color(1.0, 0.0, 0.0) });
	res.stops.push_back({ 6.0 / 6.0, Gfx::Color(1.0, 1.0, 1.0) });
	return res;
}
